/*
   ADF4001 is the cleanup PLL added to get rid of close-in jitter and coherent lines from the GNSS receiver. It is configured to have
   a very low loop bandwidth such that the ~90 Hz sidebands present on the GNSS rx timemark output  can be knocked down
   far enough to not seriously degrade the si5351 outputs when multiplied up to 2 GHz. 
   For an SDR, bench signal generator or spectrum analyzer, -90 dBC  The remaining ~20 second corrections/phase-step from the uBlox remains.

  For programming, ADF4001-1502894.pdf p 13, use the CE Pin Method "DEVICE PROGRAMMING AFTER INITIAL POWER-UP":
  Bring CE low to put the device into power-down.
  asynchronous power-down in that it happens immediately.

  Program the function latch (10).
  Program the R counter latch (00).
  Program the N counter latch (01).

  Bring CE high to take the device out of power-down. The R and
  AB counters will now resume counting in close alignment.

  Note that after CE goes high, a duration of 1 µs may be required
  for the prescaler band gap voltage and oscillator input buffer
  bias to reach steady state.

  I think that any programming of the part needs to be done completely after
  any register modifications. Learning this has been painful.

  The ADF4001 is considered an object and settings are held there but they must be also maintained by activestate so any changes must be
  updated for both objects.  I guess the ADF4001 object could simply be included in the activestate class but that hasn't yet been done.
*/
#include "ADF4001.h"
//
ADF4001::ADF4001(byte ADF4001sspin, byte ADF4001cepin, byte ADF4001muxpin, uint8_t ADF4001mode, uint32_t  speed, BitOrder ADF4001order ) {   //  Constructor
  ADF4001spi_settings = SPISettings(speed, ADF4001order, ADF4001mode) ;
}
//
//
bool ADF4001::ADF4001checkForHW() {         // set adf4001Present if SPI is able to toggle the MUX Out indicating HW presence
  uint8_t muxsave = ADF4001_R[2].getbf(4, 3);
  adf4001Present = false;
  ADF4001_R[2].setbf(4, 3, 7);            //  (start,len,value,)  MuxOut ==> DGND, lock LED off
  ADF4001writeDev(ADF4001_R[2]) ;                   //  write Function Latch (only) into HW
  delayMicroseconds(2500) ;
  if (  digitalRead(ADF4001_PIN_MUX) == LOW )  {    //  if low see if it will toggle high
    ADF4001_R[2].setbf(4, 3, 3) ;                   //  MuxOut set to VDD
    ADF4001writeDev(ADF4001_R[2]) ;                 //  forcing the HW line to VDD
    if (digitalRead(ADF4001_PIN_MUX) == HIGH)  {    // if it toggled,there must be real HW present
      adf4001Present = true;
    }
    else  adf4001Present = false;
  }
  //
  ADF4001_R[2].setbf(4, 3, muxsave);              //  restore MuxOut setting
  ADF4001writeDev(ADF4001_R[2]) ;                 //  write Function Latch (only) into HW
  delayMicroseconds(2500) ;
  return (adf4001Present);
} //  end ADF4001checkForHW
//
bool ADF4001::checkADF4001locked()  {             // non-invasive MUX Lock Detect. Returns cleanupPLL lock status
  uint8_t muxsave = ADF4001_R[2].getbf(4, 3);     // save mux state
  ADF4001_R[2].setbf( 4, 3, 1) ;                  // set MUX to use Digital lock detect( not sure if the open drain mode doesn't get applied when it shouldn't !
  ADF4001writeDev(ADF4001_R[2]);                  // write Function Latch (only) into HW
  //
  adf4001Locked  = digitalRead( ADF4001_PIN_MUX ) ;
  //
  ADF4001_R[2].setbf( 4, 3, muxsave);             //  restore mux state
  ADF4001writeDev(ADF4001_R[2]);                  //  write Function Latch (R2 only) into HW
  return adf4001Locked ;
}
//
void ADF4001::ADF4001writeDev(Reg r) {    // write out a single 24 bit register (aka 'latch' )
  byte txbyte, rxbyte ;
  int i ;
  SPI.begin();
  digitalWrite(ADF4001_PIN_SS, LOW) ;     // start the 4001 listening
  delayMicroseconds(50) ;
  //
  for ( i = 2 ; i > -1 ; i--) {           // ADF4001 registers are only 24 bits wide, 3 byte writes
    txbyte = (byte) (r.whole >> (i * 8)) ;// only bottom (shifted) 8 bits get sent out
    rxbyte = SPI.transfer(txbyte) ;       // SPI.transfer(val) sends one byte val and returns received byte (unused) from MISO if connected
  }
  digitalWrite(ADF4001_PIN_SS, HIGH) ;    // strobe the 24 bits that were sent into the ADF4001, the low-to-high transition also stops the 4001 from listening
  delayMicroseconds(500) ;
}
//
void ADF4001::ADF4001reg0ToText( ) {        // read register 0 and print it in human terms, See spec sheet Fig 24

  Serial.print("\n| -----------------------------------------------------------------------------------------------------\t |\n");
  Serial.print(    "| \t\t\t\tReference Latch (See datasheet Table III) is:\t\t\t\t |\n");

  Serial.print("| "); Serial.print(ADF4001_R[0].getbf(20, 1), BIN) ; Serial.print("\t");  // LD preceision
  Serial.print("  "); Serial.print(ADF4001_R[0].getbf(18, 2), BIN) ; Serial.print("\t");  // test mode bits

  Serial.print("  "); Serial.print(ADF4001_R[0].getbf(16, 2), BIN) ; Serial.print("\t");  // anti-backlash bits
  Serial.print("  "); Serial.print(ADF4001_R[0].getbf(2, 14), BIN) ; Serial.print("\t\t\t\t\t\t\t\t\t");  // Reference Counter

  Serial.print(" |\n");

  Serial.print("| _____\t______\t______\t______ \t       \t       \t       \t       \t       \t       \t       \t      \t\t |\n");
  Serial.print("| LDP  \tTest  \tABP   \tRef    \t       \t       \t       \t       \t       \t       \t       \t      \t\t |\n");
  Serial.print("| Prec \tBits  \tBits  \tCounter\t       \t       \t       \t       \t       \t       \t       \t      \t\t |\n");
  Serial.print("|      \t      \t      \t       \t       \t       \t       \t       \t       \t       \t       \t      \t\t |\n");

  Serial.print("| ");

  uint16_t binval;
  binval = ADF4001_R[0].getbf(20, 1);   // // LDP
  Serial.print( binval ); Serial.print("\t");

  binval = ADF4001_R[0].getbf(18, 12);   //  test mode bits
  Serial.print( binval ); Serial.print("\t");

  binval = ADF4001_R[0].getbf(16, 2);   //  anti-backlash bits
  Serial.print( binval ); Serial.print("\t");

  binval = ADF4001_R[0].getbf(2, 14);   //  Reference Counter
  Serial.print( binval ); Serial.print("\t\t\t\t\t\t\t\t\t\t");

  Serial.print(" |\n");
  Serial.print("| -----------------------------------------------------------------------------------------------------\t |\n");

}
//
void ADF4001::ADF4001reg1ToText( ) {        // read register 1 and print it in human terms, See spec sheet Fig 25

  Serial.print("\n| -----------------------------------------------------------------------------------------------------\t |\n");
  Serial.print(    "| \t\t\t\tN Counter (See datasheet Table IV) is:\t\t\t\t\t |\n");

  Serial.print(    "|  "); Serial.print(ADF4001_R[1].getbf(21, 1), BIN) ; Serial.print("\t"); // CP Gain
  Serial.print("   "); Serial.print(ADF4001_R[1].getbf(8, 13), BIN) ; Serial.print("\t");   // N Counter
  Serial.print(" \t\t\t\t\t\t\t\t\t\t |\n");

  Serial.print("| _____\t_______\t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t |\n");
  Serial.print("| CP   \t   N   \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t |\n");
  Serial.print("| Gain \tCounter\t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t |\n");
  Serial.print("|      \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t |\n");

  uint16_t binval;
  uint32_t result;
  String label;
  Serial.print("| ");
  result = ADF4001_R[1].getbf(21, 1);   // CP Gain
  Serial.print(result == 0 ? "Off  " : " On  "); Serial.print("\t");

  binval = ADF4001_R[1].getbf(8, 13);   //  N Counter
  Serial.print( binval ); Serial.print("\t\t\t\t\t\t\t\t\t\t\t\t");

  Serial.print(" |\n");
  Serial.print("| -----------------------------------------------------------------------------------------------------\t |\n");

}
//
void ADF4001::ADF4001reg2ToText( ) {        // read register 2 and print it in human terms, See spec sheet Fig 26

  Serial.print("\n| -----------------------------------------------------------------------------------------------------\t |\n");
  Serial.print(    "| \t\t\t\tFunction Latch (See datasheet Table V) is:\t\t\t\t |\n");

  Serial.print("| ");
  Serial.print("   "); Serial.print(ADF4001_R[2].getbf(21, 1), BIN) ; Serial.print("\t");   // Power Down
  Serial.print("   "); Serial.print(ADF4001_R[2].getbf(18, 3), BIN) ; Serial.print("\t");   // CP Current 2
  Serial.print("   "); Serial.print(ADF4001_R[2].getbf(15, 3), BIN) ; Serial.print("\t");   // CP Current 1
  Serial.print("   "); Serial.print(ADF4001_R[2].getbf(11, 4), BIN) ; Serial.print("\t");   // Timer Counter
  Serial.print("   "); Serial.print(ADF4001_R[2].getbf(10, 1), BIN) ; Serial.print("\t");   // Fast Lock Mode
  Serial.print("   "); Serial.print(ADF4001_R[2].getbf(9, 1), BIN) ; Serial.print("\t");    // Fast LockEnable
  Serial.print("   "); Serial.print(ADF4001_R[2].getbf(8, 1), BIN) ; Serial.print("\t");    // CP Tri-state
  Serial.print("   "); Serial.print(ADF4001_R[2].getbf(7, 1), BIN) ; Serial.print("\t");    // Phase Detector Polarity
  Serial.print("   "); Serial.print(ADF4001_R[2].getbf(4, 3), BIN) ; Serial.print("\t");    // MUX Output
  Serial.print("   "); Serial.print(ADF4001_R[2].getbf(3, 1), BIN) ; Serial.print("\t");    // Power Down 1
  Serial.print("   "); Serial.print(ADF4001_R[2].getbf(2, 1), BIN) ; Serial.print("");      // Counter Reset
  Serial.print(" \t\t\t |\n");

  Serial.print("| _____\t_______\t_______\t_______\t_______\t_______\t_______\t_______\t_______\t_______\t_______\t       \t       \t |\n");
  Serial.print("| Powr \t  CP 2 \t  CP 1 \tTimer  \tFastLck\tFastLck\t  CP   \t  PD   \t  MUX  \t  PWR  \t  CNTR \t       \t       \t |\n");
  Serial.print("| Down \t   I   \t   I   \tControl\t Mode  \tEnable \t3-State\t  Pol  \tSelect \tDown 1 \t Reset \t       \t       \t |\n");
  Serial.print("|      \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t |\n");

  uint32_t result;
  String label;
  Serial.print("| ");
  result = ADF4001_R[2].getbf(21, 1);   // Power Down
  Serial.print(result == 0 ? "Async" : "Normal"); Serial.print("\t");

  result = ADF4001_R[2].getbf(18, 3);   // CP Current 2
  Serial.print(.625 * result + .625); Serial.print("mA \t");

  result = ADF4001_R[2].getbf(15, 3);   // CP Current 1
  Serial.print(.625 * result + .625); Serial.print("mA \t");

  result = ADF4001_R[2].getbf(11, 4);   // Timer Counter
  Serial.print((3 * result + .4), 0 ); Serial.print("cyc \t");

  result = ADF4001_R[2].getbf(10, 1);   // Fast Lock Mode
  Serial.print(result == 0 ? "  Off  " : "Enabled"); Serial.print("\t");

  result = ADF4001_R[2].getbf(9, 1);   // Fast LockEnable
  Serial.print(result == 0 ? "  No   " : "Enabled"); Serial.print("\t");

  result = ADF4001_R[2].getbf(8, 0);   // CP Tri-state, 0 is non-3st normal operation
  Serial.print(result == 0 ? " Norm " : " 3ST "); Serial.print("\t");

  result = ADF4001_R[2].getbf(7, 1);   // Phase Detector Polarity
  Serial.print(result == 0 ? "  Neg" : "  Pos"); Serial.print("\t");

  result = ADF4001_R[2].getbf(4, 3);   // MUX Output
  switch (result) {
    case 0:
      label = "3State";
      break;
    case 1:
      label = "DigLck ";
      break;
    case 2:
      label = "N Dvdr ";
      break;
    case 3:
      label = "AVdd   ";
      break;
    case 4:
      label = "R Dvdr";
      break;
    case 5:
      label = "OD LD";
      break;
    case 6:
      label = "SerDat";
      break;
    case 7:
      label = "Dgnd";
      break;
    default:
      label = "Error  ";
      break;
  }
  Serial.print( label ); Serial.print("\t");

  result = ADF4001_R[2].getbf(3, 1);   // Power Down 1
  Serial.print(result == 0 ? "Normal" : "  PD2  "); Serial.print("\t");

  result = ADF4001_R[2].getbf(2, 1);   // Counter Reset
  Serial.print(result == 0 ? "Pwr ON " : "Pwr OFF"); Serial.print("\t");
  Serial.print("\t\t");

  Serial.print(" |\n");
  Serial.print("| -----------------------------------------------------------------------------------------------------\t |\n");

}
//
void ADF4001::ADF4001reg3ToText( ) {        // read register 3 and print it in human terms, See spec sheet Fig 27

  Serial.print("\n| -----------------------------------------------------------------------------------------------------\t |\n");
  Serial.print(    "| \t\t\t\tInit Latch (See datasheet Table VI) is:\t\t\t\t\t |\n");


  Serial.print("| ");
  Serial.print("   "); Serial.print(ADF4001_R[3].getbf(21, 1), BIN) ; Serial.print("\t");   // Power Down
  Serial.print("   "); Serial.print(ADF4001_R[3].getbf(18, 3), BIN) ; Serial.print("\t");   // CP Current 2
  Serial.print("   "); Serial.print(ADF4001_R[3].getbf(15, 3), BIN) ; Serial.print("\t");   // CP Current 1
  Serial.print("   "); Serial.print(ADF4001_R[3].getbf(11, 4), BIN) ; Serial.print("\t");   // Timer Counter
  Serial.print("   "); Serial.print(ADF4001_R[3].getbf(10, 1), BIN) ; Serial.print("\t");   // Fast Lock Mode
  Serial.print("   "); Serial.print(ADF4001_R[3].getbf(9, 1), BIN) ; Serial.print("\t");    // Fast LockEnable
  Serial.print("   "); Serial.print(ADF4001_R[3].getbf(8, 1), BIN) ; Serial.print("\t");    // CP Tri-state
  Serial.print("   "); Serial.print(ADF4001_R[3].getbf(7, 1), BIN) ; Serial.print("\t");    // Phase Detector Polarity
  Serial.print("   "); Serial.print(ADF4001_R[3].getbf(4, 3), BIN) ; Serial.print("\t");    // MUX Output
  Serial.print("   "); Serial.print(ADF4001_R[3].getbf(3, 1), BIN) ; Serial.print("\t");    // Power Down 1
  Serial.print("   "); Serial.print(ADF4001_R[3].getbf(2, 1), BIN) ; Serial.print("");      // Counter Reset
  Serial.print(" \t\t\t |\n");

  Serial.print("| _____\t_______\t_______\t_______\t_______\t_______\t_______\t_______\t_______\t_______\t_______\t       \t       \t |\n");
  Serial.print("| Powr \t  CP 2 \t  CP 1 \tTimer  \tFastLck\tFastLck\t  CP   \t  PD   \t  MUX  \t  PWR  \t  CNTR \t       \t       \t |\n");
  Serial.print("| Down \t   I   \t   I   \tControl\t Mode  \tEnable \t3-State\t  Pol  \tSelect \tDown 1 \t Reset \t       \t       \t |\n");
  Serial.print("|      \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t |\n");

  uint32_t result;
  String label;
  Serial.print("| ");
  result = ADF4001_R[3].getbf(21, 1);   // Power Down
  Serial.print(result == 0 ? "Async" : "Normal"); Serial.print("\t");

  result = ADF4001_R[3].getbf(18, 3);   // CP Current 2
  Serial.print(.625 * result + .625); Serial.print("mA \t");

  result = ADF4001_R[3].getbf(15, 3);   // CP Current 1
  Serial.print(.625 * result + .625); Serial.print("mA \t");

  result = ADF4001_R[3].getbf(11, 4);   // Timer Counter
  Serial.print((3 * result + .4), 0 ); Serial.print("cyc \t");

  result = ADF4001_R[3].getbf(10, 1);   // Fast Lock Mode
  Serial.print(result == 0 ? "  Off  " : "Enabled"); Serial.print("\t");

  result = ADF4001_R[3].getbf(9, 1);   // Fast LockEnable
  Serial.print(result == 0 ? "  No   " : "Enabled"); Serial.print("\t");

  result = ADF4001_R[3].getbf(8, 1);   // CP Tri-state
  Serial.print(result == 0 ? " Norm " : " 3ST "); Serial.print("\t");

  result = ADF4001_R[3].getbf(7, 1);   // Phase Detector Polarity
  Serial.print(result == 0 ? " Neg" : " Pos"); Serial.print("\t");

  result = ADF4001_R[3].getbf(4, 3);   // MUX Output
  switch (result) {
    case 0:
      label = "3State";
      break;
    case 1:
      label = "DigLck ";
      break;
    case 2:
      label = "N Dvdr ";
      break;
    case 3:
      label = "AVdd   ";
      break;
    case 4:
      label = "R Dvdr";
      break;
    case 5:
      label = "OD LD";
      break;
    case 6:
      label = "SerDat";
      break;
    case 7:
      label = "Dgnd";
      break;
    default:
      label = "Error  ";
      break;
  }
  Serial.print( label ); Serial.print("\t");

  result = ADF4001_R[3].getbf(3, 1);   // Power Down 1
  Serial.print(result == 0 ? "Normal" : "  PD2  "); Serial.print("\t");

  result = ADF4001_R[3].getbf(2, 1);   // Counter Reset
  Serial.print(result == 0 ? "Pwr ON " : "Pwr OFF"); Serial.print("\t");
  Serial.print("\t\t");

  Serial.print(" |\n");
  Serial.print("| -----------------------------------------------------------------------------------------------------\t |\n");
}
//
void  ADF4001::ADF4001setHWRegisters() {    // set HW from LOCAL register values (not necessarily activestate ones)
  // setting the SW registers from the  adf4001 class variables
  cpdisable = activestate.cpdisable;                  //  0 means current enabled
  cpcurrent = activestate.cpcurrent;
  pllMux = activestate.pllMux;
  //   RCounter = activestate.             //  activestate maintains these, may be different for GNSS or Ext discipline
  //   NCounter = activestate.

  // R0 Reference Counter Latch
  ADF4001_R[0].set(0UL) ;                 // zero the entire bit field
  ADF4001_R[0].setbf(0, 2, 0) ;           // select ADF4001_R[0]

  ADF4001_R[0].setbf(2, 14, RCounter) ;   // 14 bit (R)eference Counter, divides either GNSS timepulse or Ext10
  ADF4001_R[0].setbf(16, 2, 2) ;          // ABP Width, 2.9,1.3,6.0,2.9 ns
  ADF4001_R[0].setbf(20, 1, 1) ;          // LD Precision, 0:1 for 3:5 cycles of lock
  //*****************
  // R1 N Counter Latch
  ADF4001_R[1].set(0UL) ;                 // zero the entire bit field
  ADF4001_R[1].setbf(0, 2, 1) ;           // ADF4001_R[1} control bits

  ADF4001_R[1].setbf(8, 13, NCounter) ;   // 13 bit iNput Counter, divides the input from Clock0
  ADF4001_R[1].setbf(21, 1, 0) ;          // CP Gain
  //*****************
  // R2 Function Latch
  ADF4001_R[2].set(0UL) ;                 // zero the entire bit field
  ADF4001_R[2].setbf(0, 2, 2) ;           // control bits

  ADF4001_R[2].setbf( 2, 1, 0) ;          // counter reset
  ADF4001_R[2].setbf( 3, 1, 0) ;          // PD1 normal, use CE Pin
  ADF4001_R[2].setbf( 4, 3, pllMux) ;     // MUX function
  ADF4001_R[2].setbf( 7, 1, 0) ;          // PD polarity R(VCXO)>(GNSS)N pump down is Negative
  ADF4001_R[2].setbf( 8, 1, cpdisable) ;  // 3-state, cp output *enabled* = 0
  ADF4001_R[2].setbf( 9, 1, 0) ;          // Fastlock disabled
  ADF4001_R[2].setbf( 10, 1, 0) ;         // Fastlock mode (overruled by 9,1 = 0 )
  ADF4001_R[2].setbf( 11, 4, 8) ;         // Timer Control,  3 + (value * 4) ns
  ADF4001_R[2].setbf( 15, 3, cpcurrent) ; // CP1  0:7
  ADF4001_R[2].setbf( 18, 3, cpcurrent) ; // CP2  0:7
  ADF4001_R[2].setbf( 21, 1, 0) ;         // PD 2, with PD1=0 use CEpin
  //*****************
  // R3  Initialization Latch (same as Function Latch )
  ADF4001_R[3].set(0UL) ;                 // zero the entire bit field
  ADF4001_R[3].setbf(0, 2, 3) ;           // control bits

  ADF4001_R[3].setbf( 2, 1, 0) ;          // counter reset
  ADF4001_R[3].setbf( 3, 1, 0) ;          // PD1 normal, use CE Pin
  ADF4001_R[3].setbf( 4, 3, pllMux) ;     // 
  ADF4001_R[3].setbf( 7, 1, 1) ;          // PD polarity R>N pump down is Negative/pump up is Positive
  ADF4001_R[3].setbf( 8, 1, 0) ;          // 3-state, output enabled = 0
  ADF4001_R[3].setbf( 9, 1, 0) ;          // Fastlock disabled
  ADF4001_R[3].setbf( 10, 1, 0) ;         // Fastlock mode (overruled by 9,1 = 0 )
  ADF4001_R[3].setbf( 11, 4, 8) ;         // Timer Control,  3 + (value * 4) ns
  ADF4001_R[3].setbf( 15, 3, cpcurrent) ; // CP1 2.5 mA with 5 kohm
  ADF4001_R[3].setbf( 18, 3, cpcurrent) ; // CP2 2.5 mA with 5 kohm
  ADF4001_R[3].setbf( 21, 1, 0) ;         // PD 2, with PD1=0 use CEpin
  //
  ADF4001counterResetMethod();              //  Strobes them in and starts it all running

  //ADF4001reg1ToText( );

} // end of ADF4001setHWRegisters
//
void ADF4001::ADF4001counterResetMethod()  {  //  Write all HW Registers with Counter Reset Method, datasheet p13
  /* // Counter Reset Method datasheet p13
      Do a function latch load (10 in 2 LSBs). As part of this, load 1 to the F1 bit. This enables the counter reset.
      Do an R counter load (00 in 2 LSBs)
      Do an N counter load (01 in 2 LSBs)
      Do a function latch load (10 in 2 LSBs). As part of this, load 0 to the F1 bit. This disables the counter reset
      This sequence provides the same close alignment as the initialization method. It offers direct control over the internal reset.
      Note that counter reset holds the counters at load point and three-states the charge pump but does not trigger synchronous power-down.
      The counter reset method requires an extra function latch load compared to the initialization latch method.
  */
  ADF4001_R[3].setbf( 2, 1, 1) ;           //  1==>F1 bit, R&N Counters are held in reset
  ADF4001writeDev(ADF4001_R[2]) ;          //  write Initialization/Function Latches into HW
  ADF4001writeDev(ADF4001_R[0]) ;          //  write R Counter Latch to HW
  ADF4001writeDev(ADF4001_R[1]) ;          //  write N Counter Latch to HW
  ADF4001_R[3].setbf( 2, 1, 0) ;           //  0==>F1 bit, R&N Counters are engaged
  ADF4001writeDev(ADF4001_R[2]);           //  write Initialization/Function Latches into HW
  delay(20) ;
}
//
//
void ADF4001::ADF4001setup()  {             // initializes and sets *all* registers and HW FROM ACTIVESTATE register values.
  //  This only called once at startup. Individual registers can be temporarily modified
  //  by direct calls to ADF4001setHWRegisters(). Onus is on those calls to keep activestate current.
  //
  pinMode(ADF4001_PIN_MUX, INPUT) ;     // lock detect/MUX from the adf4001
  pinMode(ADF4001_PIN_CE, OUTPUT) ;     // PWR ON pin to the adf400
  pinMode(ADF4001_PIN_SS, OUTPUT) ;     // slave select to the adf4001
  digitalWrite(ADF4001_PIN_SS, HIGH) ;  // Don't want the ADF4001 listening until we're actually writing to it. Leave SS HIGH.
  digitalWrite(ADF4001_PIN_CE, HIGH) ;  // Power ADF4001 ON
  digitalWrite(ADF4001_PIN_SS, HIGH) ;  // Don't want the ADF4001 listening until we're actually writing to it. Leave SS HIGH.
  SPI.begin();
  //  init as a GNSS reference
  //  setting class calues from active state.
  cpcurrent = activestate.cpcurrent;        //  charge pump current
  cpdisable = activestate.cpdisable;
  pllMux    = activestate.pllMux;           //  select the MUX output
  RCounter  = activestate.GNSSpfdScaling;   //  internal ADF4001 R & N dividers
  NCounter  = RCounter;                     //  equal ref and rf pin frequencies so RCounter = NCounter
  referencePinFreq = activestate.fclock1;            //  For reporting. PCBs have VCXO wired to reference input,
  //
  ADF4001setHWRegisters();                  //  fills *all* the SW registers/latches and strobes them into HW expects class variables have been set
  //
  ADF4001checkForHW();                      //  sets adf4001Present flag, Can't do this until the part is running, I think
  //
} //  end ADF4001setup
