/*!

   @file ADF4351.cpp

   @mainpage ADF4351 Arduino library driver for Wideband Frequency Synthesizer

   @section intro_sec Introduction

   The ADF4351 chip is a wideband  synthesizer integrated circuit that can generate Frequencies
   from 35 MHz to 4.4 GHz. It incorporates a PLL (Fraction-N and Integer-N modes) and VCO, along with
   prescalers, dividers and multipiers.  The users add a PLL loop filter and reference requestedFrequency to
   create a requestedFrequency generator with a very wide range, that is tuneable in settable requestedFrequency steps.

   The ADF4351 chip provides an I2C interface for setting the device registers that control the
   requestedFrequency and output levels, along with several IO pins for gathering chip status and
   enabling/disabling output and power modes.

   The ADF4351 library provides an Arduino API for accessing the features of the ADF chip.

   The basic PLL equations for the ADF4351 are:

   \f$ RF_{out} = f_{PFD} \times (INT +(\frac{FRAC}{MOD})) \f$

   where:

   \f$ f_{PFD} = REF_{IN} \times \left[ \frac{(1 + D)}{( R \times (1 + T))} \right]  \f$

   \f$ D = \textrm{RD2refdouble, ref doubler flag}\f$

   \f$ R = \textrm{RCounter, ref divider}\f$

   \f$ T = \textrm{RD1Rdiv2, ref divide by 2 flag}\f$




   @section dependencies Dependencies

   This library uses the BigNumber library (included) from Nick Gammon

   @section author Author

   David Fannin, KK6DF

   @section license License

   MIT License

*/

#include "ADF4351.h"
#include <avr/dtostrf.h>

uint32_t ADF4351steps[] = { 10000, 20000, 50000, 100000 , 200000, 500000, 1000000, 2000000, 5000000, 10000000 }; ///< Array of Allowed Step Values (Hz)

// ADF4351 settings

ADF4351::ADF4351(byte ADF4351sspin, byte ADF4351cepin, byte ADF4351ldpin, uint8_t ADF4351mode, uint32_t  speed, BitOrder ADF4351order )  { //  modified initializer to allow changing pins for multiple instances
  ADF4351spi_settings = SPISettings(speed, ADF4351order, ADF4351mode) ;
  ADF4351pinSS = ADF4351sspin ;
  ADF4351pinCE = ADF4351cepin ;
  ADF4351pinLD = ADF4351ldpin ;

  ADF4351reffreq = UWVREFERENCE ;
  ADF4351cfreq = 0 ;
  ADF4351ChanStep = ADF4351steps[6] ;
  ADF4351Refdoubler = 0 ;
  ADF4351RCounter = 1 ;
  ADF4351Refdivider = 0 ;
  ADF4351BandSelClock = 80 ;
  ADF4351TimeoutCounter = 150 ;
  ADF4351Prescaler = 0 ;
  ADF4351MainPwrlevel = 3 ;
  SynthBPwrlevel = 3 ;
}
//
//
void ADF4351::ADF4351init(){
  pinMode(ADF4351pinSS, OUTPUT) ;
  digitalWrite(ADF4351pinSS, LOW) ;    //  prepare for SPI writes
  pinMode(ADF4351pinCE, OUTPUT) ;
  digitalWrite(ADF4351pinCE, HIGH) ;   // turn its innards on
  pinMode(ADF4351pinLD, INPUT) ;       // to read the MUX
} ;
//
//
int  ADF4351::ADF4351setf(uint32_t requestedFreq) {  //  calculate core settings from requestedFreq,  actual call to WRITE TO HW happens here

  if ( requestedFreq > ADF4351_FREQ_MAX ) return 1 ;
  if ( requestedFreq < ADF4351_FREQ_MIN ) return 1 ;

  int localosc_ratio =   2180000000UL / requestedFreq ; // note: 2180 exceeds spec by 20 MHz on low end
  ADF4351outdiv = 1 ;
  int RfDivSel = 0 ;

  // select the output divider
  while (  ADF4351outdiv <=  localosc_ratio   && ADF4351outdiv <= 64 ) {
    ADF4351outdiv *= 2 ;
    RfDivSel++  ;
  }

  if ( requestedFreq > 3600000000UL / ADF4351outdiv )
    ADF4351Prescaler = 1 ;
  else
    ADF4351Prescaler = 0 ;

  ADF4351PFDFreq = (float) ADF4351reffreq  * ( (float) ( 1.0 + ADF4351Refdoubler) / (float) (ADF4351RCounter * (1.0 + ADF4351Refdivider)));  // find the loop frequency

  BigNumber::begin(10) ;
  char tmpstr[20] ;
  
  // kludge - BigNumber doesn't like leading spaces so you need to make sure the string passed doesn't have any.
  int cntdigits = 0 ;
  uint32_t num = (uint32_t) ( ADF4351PFDFreq / 10000 ) ;
  while ( num != 0 )  {
    cntdigits++ ;
    num /= 10 ;
  }
  dtostrf(ADF4351PFDFreq, cntdigits + 8 , 3, tmpstr) ;
  // end of kludge

  BigNumber BN_PFDFreq = BigNumber(tmpstr) ;
  BigNumber BN_N = ( BigNumber(requestedFreq) * BigNumber(ADF4351outdiv) ) / BN_PFDFreq ;
  ADF4351N_Int =  (uint16_t) ( (uint32_t)  BN_N ) ;

  BigNumber BN_Mod = BN_PFDFreq / BigNumber(ADF4351ChanStep) ;
  ADF4351Mod = BN_Mod ;
  BN_Mod = BigNumber(ADF4351Mod) ;

  //  original library messed with the int, why? gives wrong answer
  //  BigNumber BN_Frac = ((BN_N - BigNumber( ADF4351N_Int)) * BN_Mod)  + BigNumber("0.5")  ;
  BigNumber BN_Frac = ((BN_N - BigNumber( ADF4351N_Int)) * BN_Mod)   ;
  ADF4351Frac = (int) ( (uint32_t) BN_Frac);

  BN_N = BigNumber( ADF4351N_Int) ;
  if ( ADF4351Frac != 0  ) {
    uint32_t gcd = ADF4351gcd_iter(ADF4351Frac, ADF4351Mod) ;
    // if possible reduce integer sizes to push spurs away from carrier
    if ( gcd > 1 ) {
      ADF4351Frac /=  gcd ;
      BN_Frac = BigNumber(ADF4351Frac) ;
      ADF4351Mod /=  gcd ;
      BN_Mod = BigNumber(ADF4351Mod) ;
    }
  }

  BigNumber BN_cfreq ;
  if ( ADF4351Frac == 0 ) {
    BN_cfreq = ( BN_PFDFreq  * BN_N) / BigNumber(ADF4351outdiv) ;
  }
  else {
    BN_cfreq = ( BN_PFDFreq * ( BN_N + ( BN_Frac /  BN_Mod) ) ) / BigNumber(ADF4351outdiv) ;
  }

  ADF4351cfreq = (uint32_t) BN_cfreq ;

  if ( ADF4351cfreq != requestedFreq ) {
    Serial.println(F("output frequency diff from requested")) ;
  }
  BigNumber::finish() ;

  if ( ADF4351Mod < 2 || ADF4351Mod > 4095) {
    Serial.println(F("ADF4351Mod out of range")) ;
    return 1 ;
  }

  if ( (uint32_t) ADF4351Frac > (ADF4351Mod - 1) ) {
    Serial.println(F("ADF4351Frac out of range")) ;
    return 1 ;
  }

  if ( ADF4351Prescaler == 0 && ( ADF4351N_Int < 23  ||  ADF4351N_Int > 65535)) {
    Serial.println(F("ADF4351N_Int out of range")) ;
    return 1;

  } else if ( ADF4351Prescaler == 1 && (  ADF4351N_Int < 75 ||  ADF4351N_Int > 65535 )) {
    Serial.println(F(" ADF4351N_Int out of range")) ;
    return 1;
  }

  // setting the registers to default values
  // R0
  R[0].set(0UL) ;                           // start all-zero, set the ones we want
  // (0,3,0) control bits
  R[0].setbf(3, 12, ADF4351Frac) ;          // fractional
  R[0].setbf(15, 16,  ADF4351N_Int) ;       // N integer

  // R1
  R[1].set(0UL) ;                           // start all-zero, set the ones we want
  R[1].setbf(0, 3, 1) ;                     // control bits
  R[1].setbf(3, 12, ADF4351Mod) ;           // Mod
  R[1].setbf(15, 12, 1);                    // phase
  R[1].setbf(27, 1, ADF4351Prescaler);      // prescaler
  // (28,1,0) phase adjust

  // R2
  R[2].set(0UL) ;                           // start all-zero, set the ones we want
  R[2].setbf(0, 3, 2) ;                     // control bits
  // (3,1,0) counter reset
  // (4,1,0) cp3 state
  // (5,1,0) power down
  R[2].setbf(6, 1, 1) ;                     // pd polarity, no inverter so = 1
  if ( ADF4351Frac == 0 )  {
    R[2].setbf(7, 1, 1) ;                   // LDP, int-n mode
    R[2].setbf(8, 1, 1) ;                   // ldf, int-n mode
  } else {
    R[2].setbf(7, 1, 0) ;                   // LDP, frac-n mode
    R[2].setbf(8, 1, 0) ;                   // ldf ,frac-n mode
  }
  R[2].setbf(9, 4, 7) ;                     // charge pump
  // (13,1,0) dbl buf
  R[2].setbf(14, 10, ADF4351RCounter) ;     //  reference counter
  R[2].setbf(24, 1, ADF4351Refdivider)  ;   //  RD1_RDiv2
  R[2].setbf(25, 1, ADF4351Refdoubler) ;    //  RD2refdouble
  R[2].setbf(26, 3, 3) ;                    //  muxout, R = 3
  // (29,2,0) low noise and spurs mode

  // R3
  R[3].set(0UL) ;                           // start all-zero, set the ones we want
  R[3].setbf(0, 3, 3) ;                     // control bits
  R[3].setbf(3, 12, ADF4351TimeoutCounter) ;// clock divider

  // (15,2,0) clk div mode (off/fastlock/srsync/rsrvd)
  // (17,1,0) reserved
  // (18,1,0) CSR
  // (19,2,0) reserved
  if ( ADF4351Frac == 0 )  {
    R[3].setbf(21, 1, 1);           //  charge cancel, reduces pfd spurs
    R[3].setbf(22, 1, 1);           //  ABP, int-n

  } else  {
    R[3].setbf(21, 1, 0) ;          //  charge cancel
    R[3].setbf(22, 1, 0);           //  ABP, frac-n
  }
  R[3].setbf(23, 1, 0) ;                    // Band Select Clock Mode
  // (24,8,0) reserved

  // R4
  R[4].set(0UL) ;                           // start all-zero, set the ones we want
  R[4].setbf(0, 3, 4) ;                     // control bits reg 4
  R[4].setbf(3, 2, ADF4351MainPwrlevel) ;   // SynthA output power 0-3 (-4dbM to 5dbM, 3db steps)
  R[4].setbf(5, 1, 1) ;                     // SynthA rf output 0:1 disable:enable
  R[4].setbf(6, 2, SynthBPwrlevel) ;	      // SynthB output power 0-3 (-4dbM to 5dbM, 3db steps)
  R[4].setbf(8, 1, 1) ;                     // SynthB rf output 0:1 disable:enable

  R[4].setbf(9, 1, 1) ;                     // SynthB Select, 0:1 divided:fundamental
  R[4].setbf(10, 1, 0)  ;                   //  Mute until Lock Detect 0:1  nomute:mute
  R[4].setbf(11, 1, 0) ;                    //  Synth vco power 0:1 up:down
  R[4].setbf(12, 8, ADF4351BandSelClock) ;  // band logic clock divider, Rcounter/BandSelClock < 125 kHz
  R[4].setbf(20, 3, RfDivSel) ;             // rf output divider select, previously calculated based on desired frequency
  R[4].setbf(23, 1, 1) ;                    // Synth feedback select  0:1  divider:VCO drives N counter
  // (24,8,0)                               //  reserved

  // R5
  R[5].set(0UL) ;                           // start all-zero, set the ones we want
  R[5].setbf(0, 3, 5) ;                     // control bits
  // (3,16,0) reserved
  R[5].setbf(19, 2, 3) ;                    // Reserved field,set to 11
  // (21,1,0) reserved
  R[5].setbf(22, 2, 1) ;                    // LD Pin Mode
  // (24,8,0) reserved

  ADF4351writeAllReg();

  return 0 ;  // ok
}
//
//
int ADF4351::ADF4351setref(uint32_t f)  { // Validate Reference
  if ( f > ADF4351_REFIN_MAX ) return 1 ;
  if ( f < 100000UL ) return 1 ;
  float newfreq  =  (float) f  * ( (float) ( 1.0 + ADF4351Refdoubler) / (float) (ADF4351RCounter * (1.0 + ADF4351Refdivider)));  // check the loop frequency
  if ( newfreq > ADF4351_PFD_MAX ) return 1 ;
  if ( newfreq < ADF4351_PFD_MIN ) return 1 ;
  ADF4351reffreq = f ;
  return 0 ;
}
//
//
void ADF4351::synthPowerUp()  {    // Power Down bit -  Note this is Reg 2
  R[2].setbf(5, 1, 0);        // (start,len,value) overwrites default bit(s) in register, but not yet strobed into part. See spec sheet for reg:bit descriptions
  ADF4351writeReg(  R[2]) ;   // then strobe it into the part.
  synthIsPowered = true ;
}
//
//
void ADF4351::synthPowerDown()  {     // Power Down whole chip (1 powers it down) -  Note this is Reg 2
  R[2].setbf(5, 1, 1);        // (start,len,value) overwrites default bit(s) in register, but not yet strobed into part. See spec sheet for reg:bit descriptions
  ADF4351writeReg(R[2]) ;     // then strobe it into the part.
  synthIsPowered = false ;
}
//
//
void ADF4351::mainOutEnable(boolean mainOut)  {     // Power Down Main OutputA, Aux output goes down with it  - Note this is Reg 4
  mainOut  ?   R[4].setbf(5, 1, 1) :  R[4].setbf(5, 1, 0);
  ADF4351writeReg(R[4]) ;     // then strobe it into the part.
  if (!mainOut) {
    auxOutEnable(false);        // Hardware forces aux off when Main is off, indicate this
  }
  mainIsPowered = mainOut ;
}
//
//
void ADF4351::auxOutEnable(boolean auxOut)  {     // Power Down Aux OutputB - Note this is Reg 4
  auxOut  ?   R[4].setbf(8, 1, 1) :  R[4].setbf(8, 1, 0);
  ADF4351writeReg(R[4]) ;                         // then strobe it into the part.
  if (auxOut)  {                                  //  turn on Main in order to get Aux
    mainOutEnable(true) ;                         //  have to turn main on to get aux power
  }
  auxIsPowered = auxOut ;
}
//
//
bool ADF4351::ADF4351locked(){
  if (digitalRead(ADF4351pinLD) == HIGH )    // dedicated pin for lock detect
    return true ;
  else
    return false ;
}
//
//
void ADF4351::ADF4351writeAllReg()  {
  int i ;
  for (i = 5 ; i > -1 ; i--) {      // write each of 6 registers, 0:5
    ADF4351writeReg(R[i]) ;      // write to the HW, one 32 bit register at a time
    delayMicroseconds(2500) ;
  }
}
//
//
void ADF4351::ADF4351writeReg( Reg r){
  byte  txbyte ;
  int i ;

  SPI.begin();
  digitalWrite(ADF4351pinSS, LOW) ; // listening now
  delayMicroseconds(10) ;
  for ( i = 3 ; i > -1 ; i--) {
    txbyte = (byte) (r.whole >> (i * 8)) ;
    SPI.transfer(txbyte) ;
  }

  digitalWrite(ADF4351pinSS, HIGH) ;  // the low-to-high transitions strobes the data and stops the 4351 from listening
  delayMicroseconds(5) ;
}
//
//
uint32_t  ADF4351::ADF4351getReg(int n) {
  return R[n].whole ;
}
//
//
uint32_t ADF4351::ADF4351gcd_iter(uint32_t u, uint32_t v) {
  uint32_t t;
  while (v) {
    t = u ;
    u = v ;
    v = t % v ;
  }
  return u ;
}
//
//
void ADF4351::ADF4351reg0ToText()  {
  // read register 0 and print it in human terms, See spec sheet Fig 24

  Serial.print("\n| -----------------------------------------------------------------------------------------------------\t |\n");
  Serial.print(    "| \t\t\t\tRegister 0 (See datasheet Fig 24) is:\t\t\t\t\t |\n");

  Serial.print("| "); Serial.print(R[0].getbf(15, 16), BIN) ; Serial.print("\t");
  Serial.print("  "); Serial.print(R[0].getbf(3, 12), BIN) ; Serial.print("\t\t\t\t\t\t\t\t\t\t\t");
  Serial.print(" |\n");

  Serial.print("| _____\t\t_______\t       \t       \t       \t       \t       \t       \t       \t       \t       \t      \t |\n");
  Serial.print("| Int  \t\tFrac-N \t       \t       \t       \t       \t       \t       \t       \t       \t       \t      \t |\n");
  Serial.print("|      \t\t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t      \t |\n");
  Serial.print("|      \t\t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t      \t |\n");

  Serial.print("| ");

  uint16_t binval;
  binval = R[0].getbf(15, 12);   // 16 bit Integer Value
  Serial.print( binval ); Serial.print("\t\t");

  binval = R[0].getbf(3, 12);   //  12 bit Integer Value
  Serial.print( binval ); Serial.print("\t\t\t\t\t\t\t\t\t\t\t");

  Serial.print(" |\n");
  Serial.print("| -----------------------------------------------------------------------------------------------------\t |\n");

} //  end reg0ToText
//
//
void ADF4351::ADF4351reg1ToText()  {
  // read register 1 and print it in human terms, See spec sheet Fig 25

  Serial.print("\n| -----------------------------------------------------------------------------------------------------\t |\n");
  Serial.print(    "| \t\t\t\tRegister 1 (See datasheet Fig 25) is:\t\t\t\t\t |\n");

  Serial.print(    "|  "); Serial.print(R[1].getbf(28, 1), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[1].getbf(27, 1), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[1].getbf(15, 12), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[1].getbf(3, 12), BIN) ; Serial.print("\t");
  Serial.print(" \t\t\t\t\t\t\t\t\t |\n");

  Serial.print("| _____\t_______\t_______\t_______\t       \t       \t       \t       \t       \t       \t       \t       \t       \t |\n");
  Serial.print("| Phase\tPrescal\tPhase  \tModulus\t       \t       \t       \t       \t       \t       \t       \t       \t       \t |\n");
  Serial.print("| Adjst\t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t |\n");
  Serial.print("|      \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t |\n");

  uint32_t result;
  String label;
  Serial.print("| ");
  result = R[1].getbf(28, 1);   // Phase Adjust
  Serial.print(result == 0 ? "Off  " : " On  "); Serial.print("\t");

  result = R[1].getbf(27, 1);   // ADF4351Prescaler
  Serial.print(result == 0 ? "  4/5  " : "  8/9  "); Serial.print("\t");

  uint16_t binval;
  binval = R[1].getbf(15, 12);   // Phase Value
  Serial.print( binval ); Serial.print("\t");

  binval = R[1].getbf(3, 12);   // Modulus Value
  Serial.print( binval ); Serial.print("\t\t\t\t\t\t\t\t\t\t");

  Serial.print(" |\n");
  Serial.print("| -----------------------------------------------------------------------------------------------------\t |\n");

} //  end reg1ToText
//
//
void ADF4351::ADF4351reg2ToText()  {
  // read register 2 and print it in human terms, See spec sheet Fig 26

  Serial.print("\n| -----------------------------------------------------------------------------------------------------\t |\n");
  Serial.print(    "| \t\t\t\tRegister 2 (See datasheet Fig 26) is:\t\t\t\t\t |\n");

  Serial.print(    "|  "); Serial.print(R[2].getbf(29, 2), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[2].getbf(26, 3), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[2].getbf(25, 1), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[2].getbf(24, 1), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[2].getbf(14, 10), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[2].getbf(13, 1), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[2].getbf(9, 4), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[2].getbf(8, 1), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[2].getbf(7, 1), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[2].getbf(6, 1), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[2].getbf(5, 1), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[2].getbf(4, 1), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[2].getbf(3, 1), BIN) ; Serial.print("");
  Serial.print(" \t |\n");

  Serial.print("| _____\t_______\t_______\t_______\t_______\t_______\t_______\t_______\t_______\t_______\t_______\t_______\t_______\t |\n");
  Serial.print("| Spur \tMux Pin\tRefernc\tRefernc\t   R   \tDouble \t  CP   \t  LDF  \t  LDP  \t  PD   \tPower  \t  CP   \tCounter\t |\n");
  Serial.print("| Nois \tOutput \tDoubler\tDivBy2 \tCounter\tBuffer \tCurrent\t  mode \t time  \tPolarty\tDown   \t3 State\tReset  \t |\n");
  Serial.print("|      \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t |\n");

  uint32_t result;
  String label;
  Serial.print("| ");
  result = R[2].getbf(29, 2);   // spur mode noise/spur 0/3
  Serial.print(result == 0 ? "Noise" : "LoSpur"); Serial.print("\t");

  result = R[2].getbf(26, 3);   // mux out, 3-st,Dvdd, Dgnd, R, N, ALD, DLD, res.
  switch (result) {
    case 0:
      label = "3State";
      break;
    case 1:
      label = "DVdd  ";
      break;
    case 2:
      label = "Dgnd  ";
      break;
    case 3:
      label = "RCount";
      break;
    case 4:
      label = "N Dvdr";
      break;
    case 5:
      label = "AnLock";
      break;
    case 6:
      label = "DigLck";
      break;
    case 7:
      label = "Error";
      break;
    default:
      label = "Error";
      break;
  }
  Serial.print( label ); Serial.print("\t");

  result = R[2].getbf(24, 1);   // ref dblr, dis/en 0/1
  Serial.print(result == 0 ? "Disabld" : "Enabled "); Serial.print("\t");

  result = R[2].getbf(24, 1);   // ref div2, dis/en
  Serial.print(result == 0 ? "Disabld" : "Enabled"); Serial.print("\t");

  result = R[2].getbf(14, 10);   // R 0-1024
  Serial.print( pow(2, result - 1), 0 ); Serial.print("\t");

  result = R[2].getbf(13, 1);   // dbl bfr, dis/en
  Serial.print(result == 0 ? "  No   " : "Bufferd"); Serial.print("\t");

  result = R[2].getbf(9, 4);   // CP I, .31 - 5 mA by .3, 0-15
  Serial.print(.313 * result + .31); Serial.print("mA \t");

  result = R[2].getbf(8, 1);   // LDF , Fracn, Int
  Serial.print(result == 0 ? " Frac-N" : "Integer"); Serial.print("\t");

  result = R[2].getbf(7, 1);   // LDP time
  Serial.print(result == 0 ? " 10 ns " : " 60 ns "); Serial.print("\t");

  result = R[2].getbf(6, 1);   // PD Pol neg/pos
  Serial.print(result == 0 ? "Negativ" : "Positiv"); Serial.print("\t");

  result = R[2].getbf(5, 1);   // Power Down dis/en
  Serial.print(result == 0 ? "Pwr ON " : "Pwr OFF"); Serial.print("\t");

  result = R[2].getbf(4, 1);   // CP 3-st dis/en
  Serial.print(result == 0 ? "Disabld" : "Enabled"); Serial.print("\t");

  result = R[2].getbf(3, 1);   // Counter Reset dis/en
  Serial.print(result == 0 ? "Disabld" : "Enabled"); Serial.print("\t");

  Serial.print(" |\n");
  Serial.print("| -----------------------------------------------------------------------------------------------------\t |\n");

} //  end reg2ToText
//
//
void ADF4351::ADF4351reg3ToText()  {
  // read register 2 and print it in human terms, See spec sheet Fig 27

  Serial.print("\n| -----------------------------------------------------------------------------------------------------\t |\n");
  Serial.print(    "| \t\t\t\tRegister 3 (See datasheet Fig 27) is:\t\t\t\t\t |\n");

  Serial.print(    "|  "); Serial.print(R[3].getbf(23, 1), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[3].getbf(22, 1), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[3].getbf(21, 1), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[3].getbf(18, 1), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[3].getbf(15, 2), BIN) ; Serial.print("\t");
  Serial.print("   "); Serial.print(R[3].getbf(3, 12), BIN) ; Serial.print("\t\t\t\t\t\t");
  Serial.print(" \t |\n");

  Serial.print("| _____\t_______\t_______\t_______\t_______\t_______\t       \t       \t       \t       \t       \t       \t       \t |\n");
  Serial.print("| Band \t  ABP  \tCharge \t Cycle \tClock  \tClock  \t       \t       \t       \t       \t       \t       \t       \t |\n");
  Serial.print("| Clock\t  time \tCancel \t Slip  \tDivMod \tDiv Val\t       \t       \t       \t       \t       \t       \t       \t |\n");
  Serial.print("|      \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t |\n");

  uint32_t result;
  String label;
  Serial.print("| ");
  result = R[3].getbf(23, 1);   // Band select  Clock mode
  Serial.print(result == 0 ? "Low  " : "High "); Serial.print("\t");

  result = R[3].getbf(22, 1);   // ABP time
  Serial.print(result == 0 ? " 6 ns  " : " 3 ns  "); Serial.print("\t");

  result = R[3].getbf(21, 1);   // charge cancel
  Serial.print(result == 0 ? "Disabld" : "Enabled"); Serial.print("\t");

  result = R[3].getbf(18, 1);   // CSR
  Serial.print(result == 0 ? "Disabld" : "Enabled"); Serial.print("\t");

  result = R[3].getbf(15, 2);   // CK div mde//
  switch (result) {
    case 0:
      label = "Div Off";
      break;
    case 1:
      label = "Fast   ";
      break;
    case 2:
      label = "Resynch";
      break;
    case 3:
      label = "Reservd";
      break;

    default:
      label = "Error  ";
      break;
  }
  Serial.print( label ); Serial.print("\t");

  uint32_t binval;
  binval = R[3].getbf(3, 12);   // clock divider
  Serial.print( binval ); Serial.print("\t\t\t\t\t\t\t\t");

  Serial.print(" |\n");
  Serial.print("| -----------------------------------------------------------------------------------------------------\t |\n");
} //  end reg3ToText
//
//
void ADF4351::ADF4351reg4ToText()  {
  // read register 4 and print it in human terms, See spec sheet Fig 28
  Serial.print("\n| -----------------------------------------------------------------------------------------------------\t |\n");
  Serial.print(    "| \t\t\t\tRegister 4 ( datasheet Fig 28) is:\t\t\t\t\t |\n");
  Serial.print(    "|  ");
  Serial.print(R[4].getbf(23, 1), BIN) ; Serial.print("\t");
  Serial.print(R[4].getbf(20, 3), BIN) ; Serial.print("\t");
  Serial.print(R[4].getbf(23, 8), BIN) ; Serial.print("\t");
  Serial.print(R[4].getbf(11, 1), BIN) ; Serial.print("\t");
  Serial.print(R[4].getbf(10, 1), BIN) ; Serial.print("\t");
  Serial.print(R[4].getbf(9, 1), BIN) ; Serial.print("\t");
  Serial.print(R[4].getbf(8, 1), BIN) ; Serial.print("\t");
  Serial.print(R[4].getbf(6, 2), BIN) ; Serial.print("\t");
  Serial.print(R[4].getbf(5, 1), BIN) ; Serial.print("\t");
  Serial.print(R[4].getbf(3, 2), BIN) ; Serial.print("\t\t\t\t");
  Serial.print(" |\n");
  // This is how to over-ride individual default bits created in setf ( must do this *after* call to setf() )
  //  R[4].setbf(9, 1, 1);      // (start,len,value) overwrites default bit(s) in register, but not yet strobed into part. See spec sheet for reg:bit descriptions
  //  writeDev(0, Synth1.R[4]) ;   // then strobe it into the part. First parameter, (int) n, is unused by the library just set to 0 here for the call
  //  Serial.print("LO1 register 4 now is "); Serial.println(Synth1.ADF4351getReg4),BIN) ;
  Serial.print("| _____\t_______\t_______\t_______\t_______\t_______\t_______\t_______\t_______\t_______\t\t\t\t |\n");

  Serial.print(    "| ");
  Serial.print("N Fbk\tuwave  \tInternl\tVCOs   \tMLTD   \tAUX    \tAUX    \tAux    \tMain   \tMain   \t");
  Serial.print(" \t\t\t |\n");

  Serial.print(    "| ");
  Serial.print("Sourc\tDivider\tLband  \tPower  \tEnable \tDivider\tEnable \tPower  \tEnable \tPower  \t\t");
  Serial.print("\t\t |\n");

  Serial.print("|      \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t       \t |\n");

  int result;   //
  Serial.print(    "| ");
  result = R[4].getbf(23, 1);   // feedback select, divided or fundamental
  Serial.print(result == 0 ? "divide" : "fund"); Serial.print("\t");
  result = R[4].getbf(20, 3);   // divider, 2^0 - 2^6
  Serial.print( pow(2, result), 0 ); Serial.print("\t");
  result = R[4].getbf(23, 8);   // internal band select, 2^0 - 2^7
  Serial.print( pow(2, result), 0 ); Serial.print("\t");
  result = R[4].getbf(11, 1);   // VCO enable, UP or DOWN
  Serial.print(result == 0 ? "UP" : "DOWN"); Serial.print("\t");
  result = R[4].getbf(10, 1);   // MTLD, mute til lock detect, drops power until lock
  Serial.print(result == 0 ? "Off" : "On"); Serial.print("\t");
  result = R[4].getbf(9, 1);   // Aux divider,
  Serial.print(result == 0 ? "divide" : "fund"); Serial.print("\t");
  result = R[4].getbf(8, 1);   // Aux Enable,can shut this second output off to save power.
  Serial.print(result == 0 ? "off" : "Enabled"); Serial.print("\t");
  result = R[4].getbf(6, 2);   // Aux Output power, 0-3, 3 dB steps from -4 dBm into 50 ohms real.
  Serial.print( -4 + (3 * result) ); Serial.print(" dBm\t");
  result = R[4].getbf(5, 1);   // Main Enable, Maybe it's main that has to be on for this to work. Not sure, read it somewhere
  Serial.print(result == 0 ? "Off" : "Enabled"); Serial.print("\t");
  result = R[4].getbf(3, 2);   // Main Output power, 0-3, 3 dB steps from -4 dBm into 50 ohms real.
  Serial.print( (int) (3 * result) - 4); Serial.print(" dBm\t");
  result = R[4].getbf(0, 3);   // Register select, always 0x04
  Serial.print("\t\t\t |\n");

  Serial.print("| -----------------------------------------------------------------------------------------------------\t |\n");

} //  end reg4ToText
//
