
#ifndef ADF4351_H
#define ADF4351_H
#include <Arduino.h>
#include <Wire.h>
#include <SPI.h>
#include <stdint.h>
#include <BigNumber.h>
#include "AAconfig.h"

#include "Reg.h"              // generic Register class for both 24 (adf4001) and 32 (adf4351) bit registers
//extern uint32_t UWVREFERENCE;
// need to use max uint32_t on arduino4351 along with Big_Number library to handle it to 1 Hz at 4 GHz
const uint32_t ADF4351_FREQ_MAX = 4294967295UL;    //  Maximum Generated Frequency = value of MAX Unsigned Long
const uint32_t ADF4351_FREQ_MIN = 34385000UL ;     //  Minimum Generated Frequency
const float ADF4351_PFD_MAX = 32000000.0;               //  Maximum Frequency for Phase Detector in Frac-N mode
const float ADF4351_PFD_MIN = 125000.0;                 //  Minimum Frequency for Phase Detector
const uint32_t ADF4351_REFIN_MAX = 320000000UL;    //  Maximum Reference Input Frequency w/o division
//
/*!
   @brief ADF4351 chip device driver

   This class provides the overall interface for an ADF4351 chip. It is used
   to define the SPI connection, initalize the chip on power up, disable/enable
   frequency generation, and set the output and reference frequencies.

   The PLL values and register values can also be set directly with this class,
   and current settings for the chip and PLL can be read.

   As a simple frequency generator, once the target frequency and desired channel step
   value is set, the library will perform the required calculations to
   set the PLL and other values, and determine the mode (Frac-N or Int-N)
   for the PLL loop. This greatly simplifies the use of the ADF4351 chip.

   The ADF4351 datasheet should be consulted to understand the correct
   register settings. While a number of checks are provided in the library,
   not all values are checked for allowed settings, so YMMV.
*/
//
class ADF4351
{
  public:

    /*!
       Constructor
       creates an object and sets the SPI parameters.
       see the Arduino SPI library for the parameter values.
       @param pin the SPI Slave Select Pin to use
       @param mode the SPI Mode (see SPI mode define values)
       @param speed the SPI Serial Speed (see SPI speed values)
       @param order the SPI bit order (see SPI bit order values)
    */

    ADF4351(byte ADF4351sspin, uint8_t ADF4351mode, uint32_t  speed, BitOrder ADF4351order ) ;
    ADF4351(byte  ADF4351spin, byte  ADF4351cepin, byte ADF4351ldpin, uint8_t ADF4351mode, uint32_t  speed, BitOrder ADF4351order ) ;

    /*!
       initialize and start the SPI interface. Call this once after instantiating the object
    */
    void ADF4351init() ;

    /*!
       sets the output frequency
       automatically calculates the PLL and other parameters
       returns false if the desired frequency does not match the
       calculated frequency or is out of range.

       @param freq target frequency
       @return success (True or False)
    */
    int ADF4351setf(uint32_t requestedFreq) ; // set freq

    /*!
       sets the reference frequency
       sets the incoming reference frequency to the ADF4351 chip,
       based on your OXCO or other freqeuncy reference source value.
       Checks for min and max settings

       @param f reference frequency
       @return success (True or False)
    */
    int ADF4351setref(uint32_t f) ;  // set reference freq

    /*!
       turns on the output frequency (enables the CE pin)
       The chip is still powered up, so all settings are
       maintained in this mode.
    */
    void synthPowerUp();

    /*!
       turns off the output frequency (disables the CE pin)
       The chip is still powered up, so all settings are
       maintained in this mode.
    */
    void synthPowerDown();

    /*!
       controls auxiliary/divided output
    */
    void  mainOutEnable(boolean state);
    /*!
       controls auxiliary/divided output
    */
    void  auxOutEnable(boolean state);
    /*
    */
    void ADF4351writeAllReg();
    /*!
       writes the register value to the device
       normally used as an internal helper function
       @param n nth register to write to
       @param r the register value to write
    */
    void ADF4351writeReg( Reg r) ;

    /*!
       gets the value of the device register
       @param n nth register on the device
       @return the current value of the register
    */
    uint32_t ADF4351getReg(int n) ;

    /*!
       calculates the greatest common denominator for two values
       helper function to calculate PLL values
       @param u value 1
       @param v value 2
       @return the greatest common denominator
    */
    uint32_t ADF4351gcd_iter(uint32_t u, uint32_t v) ;

    /*!
      Returns bool of the Lock Detect pin
    */
    bool ADF4351locked();
    /*!
       stores the SPI settings
    */
    SPISettings ADF4351spi_settings;

    /*!
       Dump registers in human readable form
    */
    void ADF4351reg0ToText();
    void ADF4351reg1ToText();
    void ADF4351reg2ToText();
    void ADF4351reg3ToText();
    void ADF4351reg4ToText();
    //   void ADF4351writeReg();

    /*!
       stores the SPI Slave Select Pin
    */
    uint8_t ADF4351pinSS ;

    /*!
      stores the chip enable Pin
    */
    uint8_t ADF4351pinCE ;

    /*!
       stores the  lock detect Pin
    */
    uint8_t ADF4351pinLD ;
    /*!
       array for storing the working register values (used for writing)
    */
    Reg R[6] ;            // registers 0 - 5

    /*!
       stores the reference frequency
    */
    uint32_t ADF4351reffreq;

    /*!
       stores the current frequency generation on/off status
    */
    boolean synthIsPowered ;
    /*!
      stores the  on/off status of the AuxOutput
    */
    boolean mainIsPowered ;
    /*!
       stores the  on/off status of the AuxOutput
    */
    boolean auxIsPowered ;
    /*!
       stores the calculated frequency (vs the desired frequency)
       used to check for issues in the setf() function.
       this value is overwritten each time sef() is called.
    */
    uint32_t ADF4351cfreq ;

    /*!
       stores the PLL INT value for the current frequency
       this value is overwritten each time setf() is called.
    */
    uint16_t ADF4351N_Int ;
    /*!
       The PLL Frac value for the current frequency
       this value is overwritten each time sef() is called.
    */
    int ADF4351Frac ;
    /*!
       The PLL Mod value for the current frequency
       this value is overwritten each time sef() is called.
    */
    uint32_t ADF4351Mod ;

    /*!
       The PLL Phase Detect Freq  value for the current frequency
       can be changed if a new reference frequency is used.
    */
    float ADF4351PFDFreq ;

    /*!
       the channel step value
       can be directly changed to set a new frequency step
       from the defined steps[] array. You should not use
       an arbitrary value for this.
    */
    uint32_t ADF4351ChanStep;

    /*!
       the PLL output divider value for the current frequency
       this value is overwritten each time sef() is called.
    */
    int ADF4351outdiv ;
    /*!
       the PLL ref freq doubler flag for the current frequency
       it is used to double the incoming ref frequency.
       this should used to when setting up the reference frequency
       set to 0 or 1
    */
    uint8_t ADF4351Refdoubler ;

    /*!
       the PLL R counter value for the current frequency
       10bit counter used to divide the ref freq for the PFD
    */
    int ADF4351RCounter ;

    /*!
       the PLL ref freq Divider by 2
       sets a divide-by-2 between the Rcounter and PFD
    */
    uint8_t ADF4351Refdivider ;

    /*!
       the PLL Band Select Clock Value
    */
    uint8_t ADF4351BandSelClock ;
    /*!
       the PLL Clock Divider value
       the 12bit timeout counter for activation of phase resync and fast lock.
    */
    int ADF4351TimeoutCounter ;

    /*!
       The PLL ref freq prescaler (divider) flag
       set to 0 or 1
    */
    uint8_t ADF4351Prescaler ;

    /*!
       The power output level settings
       allowed values 0-4
    */
    byte ADF4351MainPwrlevel ;
    byte SynthBPwrlevel ;

};


#endif
