/*
  Time Pulse Parameters - Frequency
  By: Paul Clark (PaulZC)
  Date: January 13th, 2021

  License: MIT. See license file for more information but you can
  basically do whatever you want with this code.

  The this was borrowed from showed how to change the time pulse parameters and configure the TIMEPULSE (PPS)
  pin to produce a 1kHz squarewave
*/
#include "GNSSTime.h"
//
// Create storage for the time pulse parameters
UBX_CFG_TP5_data_t timePulseParameters;
//
//
uint8_t numSats() { // return number of acquired satellites
  uint8_t numSvs = ( ubxDataStruct->header.numSvs);
  return numSvs;
}
//
//
// Callback: newNAVSAT will be called when new NAV SAT data arrives
// See u-blox_structs.h for the full definition of UBX_NAV_SAT_data_t
//         _____  You can use any name you like for the callback. Use the same name when you call setAutoNAVSATcallback
//        /             _____  This _must_ be UBX_NAV_SAT_data_t
//        |            /                _____ You can use any name you like for the struct
//        |            |               /
//        |            |               |
void newNAVSAT(UBX_NAV_SAT_data_t *ubxDStruct)   //  Callback from uBlox
{
  ubxDataStruct = ubxDStruct;
}
//
//
boolean GNSSsetup() {   //  returns true if have timepulse capability
  boolean  haveuBloxComms;
  const uint32_t UBLOX_CFG_I2C_ENABLED = 0x10510003;  // Flag to indicate the I2C interface should be enabled (not used yet)
  Serial1.begin(38400);             // instantiate UART#1 on IoT33
  //myGNSS.enableDebugging();       // Uncomment this line to enable debug messages
  myGNSS.setFileBufferSize(32);     //  Wire library uses 32 bit buffer so 32 bytes should be enough, This must be called _before_ .begin.

  if (myGNSS.begin() == true)   {   //  Try  I2C  connect to the u-blox module using I2C/Wire port and pins #'s defined in AAconfig.h
    Serial.print("Found a GNSS rx module via I2C.");
    haveuBloxComms = true;
  }
  else  {                           // no I2C, try Serial1 UART to uBlox a few times
    int tries = 0;
    int start = millis();
    do {                            // try to connect via IoT UART#1
      Serial1.begin(38400);
      if (myGNSS.begin(Serial1) == true) {
        Serial.print("Have a serial port for UART#1");
        haveuBloxComms = true;
        break;
      }
      //
      { // give up after a few tries
        tries++;
        if (tries > 3) {
          Serial.print("No uBloxComms !");
          haveuBloxComms = false;
          break;
        }
      }
    } while (true);                      // we break out of this with haveuBloxComms set one way or the other
  }
  //
  if (haveuBloxComms) {                     // can continue because it's talking to us somehow (serial or I2C)
    const int waittime = 5000;              //  Don't exceed watchdog timer here !
    //
    //      if (activestate.gnssReset) {          //  request to completely reset the GNSS module once/FWupload or if directed by user from Status page
    //        myGNSS.factoryDefault();
    //        delay(waittime);                    // Reset everything and wait while the module restarts
    //        Serial.print(" Resetting GNSS module to factory default ");
    //        Serial.print(" Totaly remove and reapply power to restart");
    //        activestate.gnssReset = false;      //  just do it once, either on firstrun with default set true or if requested by user
    //        writeStateToEEPROM(defaultstate);
    //        while(true){};
    //      }
    //
    Serial.println("");
    // check to see what FW version the uBlox has in it
    Serial.print(F("uBlox Protocol Version: "));
    byte versionHigh = myGNSS.getProtocolVersionHigh();
    Serial.print(versionHigh);
    Serial.print(".");
    byte versionLow = myGNSS.getProtocolVersionLow();
    Serial.print (versionLow);
    Serial.print(" ");
    //
    if (versionHigh > 0) {                                      //  uBlox is actually talking to us
      //  Can support different protoocols/Series here if we want
      if (versionHigh > 20)    {
        Serial.println("(newer)");
      }
      else   {
        Serial.println("old protocol ");
      }
      //  turn off NMEA noise.  Might want to turn it on later if msg edge,lat/lon or satellite time info needed
      myGNSS.setI2COutput(COM_TYPE_UBX);                        //  Set I2C to output UBX only
      myGNSS.setUART1Output(COM_TYPE_UBX);                      //  Set  UART port to output UBX only
      myGNSS.saveConfigSelective(VAL_CFG_SUBSEC_IOPORT);        //  Save (only) the communications port settings to flash and BBR
      /*  The GNSS identifiers - used by UBX-CFG-GNSS (0x06 0x3E) GNSS system configuration
        enum sfe_ublox_gnss_ids_e
        {
        SFE_UBLOX_GNSS_ID_GPS,
        SFE_UBLOX_GNSS_ID_SBAS,
        SFE_UBLOX_GNSS_ID_GALILEO,
        SFE_UBLOX_GNSS_ID_BEIDOU,
        SFE_UBLOX_GNSS_ID_IMES,
        SFE_UBLOX_GNSS_ID_QZSS,
        SFE_UBLOX_GNSS_ID_GLONASS
        };
        // Enable/Disable individual GNSS systems using UBX-CFG-GNSS
        //  bool SFE_UBLOX_GNSS::enableGNSS(bool enable, sfe_ublox_gnss_ids_e id, uint16_t maxWait)
      */
      myGNSS.enableGNSS(false, SFE_UBLOX_GNSS_ID_SBAS, 250);    //  Don't want SBAS running - per uBlox recommendation for timepulse
      myGNSS.enableGNSS(false,  SFE_UBLOX_GNSS_ID_QZSS, 250);   //  no QZSS, in Western Hemisphere anyway
      myGNSS.enableGNSS(gnssState.GPS,  SFE_UBLOX_GNSS_ID_GPS, 250);
      myGNSS.enableGNSS(gnssState.GLONASS,  SFE_UBLOX_GNSS_ID_GLONASS, 250);
      myGNSS.enableGNSS(gnssState.GALILEO,  SFE_UBLOX_GNSS_ID_GALILEO, 250);
      myGNSS.enableGNSS(gnssState.BEIDOU,  SFE_UBLOX_GNSS_ID_BEIDOU, 250);
      //
      //  the following are available to reduce discontinuity in timepulse -   maybe
      /*
         This message allows the user to alter the rate at which navigation solutions (and the
        measurements that they depend on) are generated by the receiver. The calculation of the
        navigation solution will always be aligned to the top of a second zero (first second of the
        week) of the configured reference time system.
        (Navigation period is an integer multiple of the measurement period in protocol versions
        greater than 17)
      */
      uint32_t navPeriod = 250;                   // navigation calculation period  ms
      myGNSS.setNavigationFrequency(navPeriod);   // Produce one solution per [arg] measurements
      //      Serial.print( F("navigation Setting = ") ); Serial.print(navPeriod ); Serial.print(F(""));

      uint32_t measPeriod = 1;                      //  measurment period in navPeriod s
      myGNSS.setMeasurementRate(measPeriod,  1100); //  measure rate in navPeriods, (wait <1.1 seconds after setting)
      //      Serial.print( F(" \tmeasurement Setting = ") ); Serial.print(measPeriod * navPeriod);  Serial.print(F(" ms\n"));

      //
      myGNSS.setAutoNAVSATcallbackPtr(&newNAVSAT);               //  Enable automatic NAV SAT messages with callback to newNAVSAT
      //
      // Get & set the time pulse parameters
      if (myGNSS.getTimePulseParameters(&timePulseParameters) == false)    {    //  wrong version or wrong model?
        Serial.println(F("couldn't get TimePulseParameters, absent or wrong uBlox model?"));
        tpCapableGNSSPresent = false;  // GNSS useless to us  OR    uBlox stuck in a state and needs a power cylce!  **TODO**
      }
      else {                                                      // timepulse configuration is described in GNSS.G7-SW-12001 section 12.3
        // Print the CFG TP5 version
        Serial.print(F("UBX_CFG_TP5 version: "));
        Serial.print(timePulseParameters.version);
        Serial.println(F(", configuring timepulse"));
        tpCapableGNSSPresent = true;
        //
        // Now configure the timepulse for both sat unlocked and locked conditions
        timePulseParameters.tpIdx = 0;                              // Select the TIMEPULSE pin

        // Configure the time pulse pin to produce a defined frequency (not period because that uses microsecs which is capped at 1 us => 1 MHz )
        // While the module is *not sat locked* to GNSS time, make it generate logic LOW on timepulse pin to use as LOCK DETECT
        timePulseParameters.freqPeriod = 1;                         // Set the frequency/period to 1 pps
        timePulseParameters.pulseLenRatio = 0x00000000;             // Set the no-satellite-lock pulse ratio to 0%, OFF
        // Set the satellite-locked pulse ratio to 100%, ON
        // so becomes a satellite lock detect. Change this duty cycle later to get a square wave tp
        timePulseParameters.freqPeriodLock =  UBLOX_MAGIC_FREQ   ;  // magic number so uBlox won't slip cycles and glitch, selected in aaconfig.h
        timePulseParameters.pulseLenRatioLock = 0xFFFFFFFF;         // initially set the the pulse ratio to as near 1.0 mark:space *during lock* as we can

        timePulseParameters.flags.bits.active = 1;                  // Make sure the active flag is set to enable the time pulse. (Set to 0 to disable.)
        timePulseParameters.flags.bits.lockedOtherSet = 1;          // freqPeriod while locking and freqPeriodLock when locked to GNSS time
        timePulseParameters.flags.bits.isFreq = 1;                  // set the frequency (not the period)
        timePulseParameters.flags.bits.isLength = 0;                // pulseLenRatio is a ratio / duty cycle (* 2^-32) - not a length (in us)
        timePulseParameters.flags.bits.polarity = 1;                // rising edge at the top of second. (Set to 0 for falling edge.)

        // Now write out the time pulse register parameters to HW
        myGNSS.setTimePulseParameters(&timePulseParameters);        //  write the registers (should never fail)

      } // end  timepulseconfigured
    } // end versionHigh> 0
    else {
      Serial.print(F("uBlox version = 0!\n"));
      tpCapableGNSSPresent = false;
    }
  }   // end haveuBloxComms
  return (tpCapableGNSSPresent);
} // end GNSSsetup()
//
//
boolean checkSatlocked()  {                               // non-invasive setting timePulsePresent & satLocked
  int tp = averageBitRead(TPDETECT);                      // analogRead of what is essentially a digital imepulse & satellite lock detect
  float tpV = ((float) tp / 4096.0) * calcArefV;          // compare in Volts to take care of all the references and such
  if (tpV > (.20 * VDD))  {                               // Higher than low-logic so must be sat Locked or timepulse
    satLocked = true;                                     // locked to satellites, maybe timepulse if present too
    if (tpV > (.70 * VDD))  {
      timePulsePresent = false;
    }
    else timePulsePresent = true;
  }
  else {
    satLocked = false;
    timePulsePresent = false;
  }
  return (satLocked);
}
//
boolean setTimepulsePinIsReference(boolean enableTimepulse) {     // enable  timepulse (present if satellites are locked)
  /*   in conjunction with uBLox config & checkSatlocked(), enables timepulse
    Usage: initially configure uBlox then when checkSatlocked() == true call this to turn on the squarewave,
  */
  //
  //
  if (enableTimepulse) {                                    // enable timepulse
    timePulseParameters.pulseLenRatioLock = 0x80000000;     // enable timepulse, duty cycle = 50%
    myGNSS.setTimePulseParameters(&timePulseParameters);    // and write it to the HW
    delay(100);                                             // giving it time to settle
    timePulsePresent = true ;
  }
  else {
    timePulseParameters.pulseLenRatioLock = 0x00000000;     // disable timepulse & pin, duty cycle = 0%
    myGNSS.setTimePulseParameters(&timePulseParameters);    // and write it to the HW
    delay(100);                                             // giving it time to settle
    timePulsePresent = false ;
  }
  return (timePulsePresent);
}
