/*
   si5351c.cpp - Si5351 library for Arduino

   Copyright (C) 2015 - 2019 Jason Milldrum <milldrum@gmail.com>
                             Dana H. Myers <k6jq@comcast.net>

   Some tuning algorithms derived from clk-si5351c.c in the Linux kernel.
   Sebastian Hesselbarth <sebastian.hesselbarth@gmail.com>
   Rabeeh Khoury <rabeeh@solid-run.com>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdint.h>
#include "Arduino.h"
#include "avr/dtostrf.h"

#include "Wire.h"

#include "si5351.h"
#include "AAconfig.h"
#include "BigNumber.h"
void setClock0(uint32_t fset );
void printStatusAssignsPllfreq();
void printClkCtrlSettings( uint8_t clk );
void printBits(uint8_t reg_val);
//
//extern const boolean COMPILED_FOR_REFERENCE;
extern struct state defaultstate, activestate ;
extern boolean si5351Present;

boolean CF( uint32_t reqFreq,  uint64_t pllFreq);

Si5351 si5351c;

//  setClocks take Hz and delivers it as ULL to library  si5351_write AFTER set_freq is a workaround to repair set+freq:set_ms clobbering the corresponding
//  clock_control register if/when they read wrong on an IoT33.

void setClock0(uint32_t fset )  { //  First to write
  si5351c.set_freq(  (uint64_t) fset  * SI5351_FREQ_MULT,    SI5351_CLK0); 
  activestate.fclock0 = fset;
  
  si5351c.si5351_write( 16, 0x0E );  //  setting to +11 dBm always
  activestate.pwr[0] = 0b10;
}

void setClock1(uint32_t fset )  { //
  si5351c.set_freq(  (uint64_t) fset  * SI5351_FREQ_MULT,  SI5351_CLK1);
  activestate.fclock1 = fset;

  si5351c.si5351_write( 17, 0x0C | activestate.pwr[1] );
}

void setClock2(uint32_t fset )  { //  w/ PLLA = 600, (22+2/9) gives exact 27
  si5351c.set_freq(  (uint64_t) fset  * SI5351_FREQ_MULT,   SI5351_CLK2);
  activestate.fclock2 = fset;
  si5351c.si5351_write( 18, 0x0C | activestate.pwr[2] );
}

void setClock3(uint32_t fset )  { //
  si5351c.set_freq( (uint64_t) fset  * SI5351_FREQ_MULT,  SI5351_CLK3);
  activestate.fclock3 = fset;
  si5351c.si5351_write( 19, 0x0C | activestate.pwr[3] );
}

void setClock4(uint32_t fset )  { //
  si5351c.set_freq( (uint64_t) fset  * SI5351_FREQ_MULT, SI5351_CLK4);
  activestate.fclock4 = fset;
  si5351c.si5351_write( 20, 0x0C | activestate.pwr[4] );
}

void setClock5(uint32_t fset )  { //
  si5351c.set_freq( (uint64_t) fset  * SI5351_FREQ_MULT, SI5351_CLK5);
  activestate.fclock5 = fset;

  si5351c.si5351_write( 21, 0x0C | activestate.pwr[5] );
}

void setClock6(uint32_t fset )  { //  First to write - owns pllb and is master of clock7
//  activestate.fclock7 = 0;
  si5351c.set_freq( (uint64_t) fset  * SI5351_FREQ_MULT,  SI5351_CLK6);
  activestate.fclock6 = fset;

  si5351c.si5351_write( 22, 0x2C | activestate.pwr[6] );  //  PLLB  and clkIN
}

void setClock7(uint32_t fset )  { //

  si5351c.set_freq( (uint64_t) fset  * SI5351_FREQ_MULT,  SI5351_CLK7)  ?   activestate.fclock7 = 0 :   activestate.fclock7 = fset;

  si5351c.si5351_write( 23, 0x2C | activestate.pwr[7] );  //  ...PLLB  and clkIN
}

void printBits(uint8_t reg_val)  {
  for (uint8_t b = 7; b < 0xFF ; b--)  { // un-suppress leading zeroes
    (reg_val >> b) & 0b1  ?   Serial.print("1") : Serial.print("0");
  }
}

void printClkCtrlSettings( uint8_t clk )  { //  pass clock #  to read
  uint8_t reg_val = 0;
  reg_val = si5351c.si5351_read(SI5351_CLK0_CTRL + clk );

  Serial.print("from HW: clk"); Serial.print(clk); Serial.print(" ");
  (reg_val & 0x80) >> 7 == 1 ? Serial.print ("PDN ") : Serial.print("PUP ") ;
  (reg_val & 0x40) >> 6 == 1 ? Serial.print ("INT  ")  : Serial.print("FRAC ") ;
  (reg_val & 0x20) >> 5 == 1 ? Serial.print ("pllB ") : Serial.print("pllA ") ;
  (reg_val & 0x10) >> 4 == 1 ? Serial.print ("INV  ") : Serial.print("NORM ") ;
  //
  if ( (reg_val & 0x0C) == SI5351_CLK_INPUT_MULTISYNTH_N)   Serial.print ("MSn  ");
  if ( (reg_val & 0x0C) == SI5351_CLK_INPUT_MULTISYNTH_0_4)   Serial.print ("0MS00");
  if ( (reg_val & 0x0C) == SI5351_CLK_INPUT_CLKIN)   Serial.print ("CKIN ");
  if ( (reg_val & 0x0C) == SI5351_CLK_INPUT_XTAL)   Serial.print ("XTAL ");
  //
  if ( (reg_val & 0x03) == SI5351_DRIVE_8MA)     Serial.print ("8mA") ;
  if ( (reg_val & 0x03) == SI5351_DRIVE_6MA)     Serial.print ("6mA") ;
  if ( (reg_val & 0x03) == SI5351_DRIVE_4MA)     Serial.print ("4mA") ;
  if ( (reg_val & 0x03) == SI5351_DRIVE_2MA)     Serial.print ("2mA") ;

  Serial.print("\t ");
  printBits(reg_val);

  Serial.println("") ;
}

void printStatus( ) { //  update, read and print reg0
  si5351c.update_status();  delay(100);
  Serial.print("SYS_INIT: ");
  Serial.print(si5351c.dev_status.SYS_INIT);
  Serial.print("  LOL_A: ");
  Serial.print(si5351c.dev_status.LOL_A);
  Serial.print("  LOL_B: ");
  Serial.print(si5351c.dev_status.LOL_B);
  Serial.print("  LOS: ");
  Serial.print(si5351c.dev_status.LOS);
  Serial.print("  REVID: ");
  Serial.println(si5351c.dev_status.REVID);
}

void printStatusAssignsPllfreq()  {
  Serial.println();
  printStatus( );
  Serial.print ("from memory not HW:");
  Serial.print("PLL_A = ");  Serial.print(si5351c.plla_freq / 1e8, 3);
  Serial.print("\tPLL_B = ");  Serial.println(si5351c.pllb_freq / 1e8, 3);

  for (uint8_t i = 0; i < 8; i++)
  { Serial.print("CLK"); Serial.print(i, 1); Serial.print(" = "); Serial.print(si5351c.clk_freq[i] / 1e8, DEC);
    Serial.print("\tPLL_");  Serial.println(si5351c.pll_assignment[i] == 0 ? "A" : "B");
  }

  Serial.println();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

boolean si5351cSetup()  {
  uint8_t reg_val;

  /*  xo_freq !=0 gets corrected, xo_freq==0 gets set from SI5351_XTAL_FREQ.
    //  each si5351 output clock has a choice of 4 si5351_clock_sources:
    //  one of {SI5351_CLK_SRC_XTAL (p3 of a -C part), SI5351_CLK_SRC_CLKIN (p6 of a C part), SI5351_CLK_SRC_MS0, SI5351_CLK_SRC_MS}
    //  for si5351_clock_source==SI5351_CLK_SRC_CLKIN,    SI5351_CLKIN_DIV_1  is always used so xo_freq is used directly.
    //  in the case of SI5351_CLK_SRC_XTAL  this value may be divided using SI5351_CLKIN_DIV_x as necessary
    //  in the case of SI5351_CLK_SRC_MS0 or SI5351_CLK_SRC_MS the clocking comes from an internal msdivider so no division is relevant

    //  The resulting value is then used by set_correction():set_pll(plla_freq, SI5351_PLLx) to tweak *both* public plla_freq and pllb_freq
    //  frequencies from the single SI5351_PLL_INPUT_XO value (stored in private class variable int32_t ref_correction[ ].)
    //  At this point reference setup is finished.
    //   next reset() is called which:
    //  * toggles power down/up on each output clock,
    //  * sets both PLL frequencies 
    //  * sets the ms source for each clock output to either PLLA or PLLB,
    //  * assigns each class pll_assignment[] to either PLLA or PLLB, mirroring the previous ms source setting
    //  * resets the VCXO parameters (usef in B version),
    //  * resets both PLLs.
    //  * sets every clock output:     clk_freq[i] = 0;output_enable((enum si5351_clock)i, 0); clk_first_set[i] = false;
    //
  */
 
 si5351c.init(SI5351_CRYSTAL_LOAD_C, 0, 0) ;
 
//    {
//      si5351c.update_status();
//      delay(10);
//    } while (Serial.print(si5351c.dev_status.SYS_INIT) != 0 )
// Serial.print(si5351c.dev_status.SYS_INIT)  ;

  
    EXTIN ?  si5351c.si5351_write( 15, 0x0C )  : si5351c.si5351_write( 15, 0x00)   ;     //  select reference pin.  do this in RESET()?
    si5351c.pll_reset(SI5351_PLLA);
    si5351c.pll_reset(SI5351_PLLB);
    
    setClock0(freq10M );  //  First to write should set an unset PLLA, I think
  
    if (COMPILED_FOR_REFERENCE)
      setClock1(activestate.fclock1 );
    else   {
      activestate.fclock1 = freq25M;
      setClock1(activestate.fclock1 );
    }
  
    setClock2(activestate.fclock2);      // As first to write > freq100M, I think CLK2 claims and sets PLLB here
  
    setClock3(activestate.fclock3);
  
    setClock4(activestate.fclock4);
  
    setClock5(activestate.fclock5);
  
    setClock6(activestate.fclock6);
  
    setClock7(activestate.fclock7); 
    return (true); 

} //  end si5351cSetup()
//
